<?php

namespace Drupal\entity_fallback_value\Generators\EntityFallbackValuePlugin;

use Drupal\Core\Extension\ModuleHandlerInterface;
use DrupalCodeGenerator\Command\BaseGenerator;
use DrupalCodeGenerator\Utils;
use Symfony\Component\Console\Question\Question;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class ConfigEntityClonerProcessorPluginGenerator.
 *
 * @package Drupal\config_entity_cloner\Generators
 */
class EntityFallbackValuePluginGenerator extends BaseGenerator {

  /**
   * The command name.
   *
   * @var string
   */
  protected $name = 'entity-fallback-plugin';

  /**
   * The description.
   *
   * @var string
   */
  protected $description = 'Generates a EntityFallbackValues plugin.';

  /**
   * The alias.
   *
   * @var string
   */
  protected $alias = 'efp';

  /**
   * The template dir path.
   *
   * @var string
   */
  protected $templatePath = __DIR__;

  /**
   * Module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * ConfigEntityClonerProcessorPluginGenerator constructor.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param string $name
   *   Then name.
   */
  public function __construct(ModuleHandlerInterface $moduleHandler = NULL, $name = NULL) {
    parent::__construct($name);
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * {@inheritdoc}
   */
  protected function interact(InputInterface $input, OutputInterface $output) {
    $questions = [];
    $questions['machine_name'] = new Question('Module machine name');
    $questions['machine_name']->setValidator([
      Utils::class,
      'validateMachineName',
    ]);

    // Plugin name.
    $questions['plugin_name'] = new Question('Plugin Name');
    $questions['plugin_name']->setValidator([Utils::class, 'validateRequired']);

    // Plugin id.
    $plugin_id = function ($vars) {
      return Utils::camel2machine($vars['plugin_name']);
    };
    $questions['plugin_id'] = new Question('Plugin id', $plugin_id);

    $questions['applies_on'] = new Question('Applies on (separate by comma) ? Ex: "node","taxonomy_term.voc_1" ');
    $questions['applies_on']->setValidator([Utils::class, 'validateRequired']);

    $this->collectVars($input, $output, $questions);

    // Init variables variantes.
    $this->vars['plugin_id'] = Utils::human2machine($this->vars['plugin_id']);
    $this->vars['plugin_class'] = Utils::camelize($this->vars['plugin_id']);
    $this->initAppliesOn();

    // Generateplugin.
    $this->generatePlugin();
  }

  /**
   * Generate plugin.
   */
  protected function generatePlugin() {
    // Création du plugin.
    $this->addFile()
      ->path('src/Plugin/entity_fallback_value/' . $this->vars['plugin_class'] . '.php')
      ->template('templates/plugin.twig');
  }

  /**
   * Init applies on.
   */
  protected function initAppliesOn() {
    $this->vars['applies_on'] = array_filter(explode(',', $this->vars['applies_on']));
    foreach ($this->vars['applies_on'] as &$value) {
      $value = '"' . trim($value) . '"';
    }
    $this->vars['applies_on'] = implode(',', $this->vars['applies_on']);
  }

}
