<?php

namespace Drupal\entity_fallback_value\PluginManager\Plugin;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\entity_fallback_value\Traits\AccessNestedFieldsTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Gives applies base method for EntityFallback plugins.
 *
 * @package Drupal\entity_fallback_value\PluginManager\Plugin
 */
abstract class AbstractEntityFallbackValuePlugin implements ContainerFactoryPluginInterface, EntityFallbackValuePluginInterface {
  use AccessNestedFieldsTrait;

  /**
   * AppliesOn configuration.
   *
   * @var array
   */
  protected array $appliesOn;

  /**
   * AbstractEntityFallbackValuePlugin constructor.
   *
   * @param array $appliesOn
   *   The configuration for default apply.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   */
  public function __construct(array $appliesOn, EntityRepositoryInterface $entityRepository) {
    $this->appliesOn = $appliesOn;
    $this->setEntityRepository($entityRepository);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $container->get('entity.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function applies(ContentEntityInterface $contentEntity = NULL): bool {
    $applies = FALSE;
    $entityTypeId = $contentEntity->getEntityTypeId();
    $bunde = $contentEntity->bundle();
    if (isset($this->appliesOn[$entityTypeId])) {
      // Applies on every bundle.
      if (empty($this->appliesOn[$entityTypeId])) {
        $applies = TRUE;
      }
      elseif (in_array($bunde, $this->appliesOn[$entityTypeId])) {
        $applies = TRUE;
      }
    }

    return $applies;
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityFallbackValues(ContentEntityInterface $contentEntity = NULL) {
    return $this->getEntityFallbackValuesFromDefintions($contentEntity, $this->getEntityFallbackDefinitions());
  }

}
